const Times = require("../public/times");
const Init = require("./init");

/**
 * earliest date : 2024-11-26 00:00:00
 */
class Stats extends Init {
  constructor(coin) {
    const method = "stats";
    super(coin, method);
  }

  async query_pooldb_mhs24h(date) {
    try {
      const yMd = Times.utcTime(new Date(date).valueOf() - 1000 * 60 * 60 * 24).split(" ")[0];
      const table_date = yMd.replace(/\-/g, "");
      let table = `${this.coin}_miners_stats_${table_date}`;
      console.log(table);

      const sql = `SELECT user, SUM(mhs24h) AS mhs24h FROM ${table} WHERE date >= ? AND date <= ? GROUP BY user;`;
      const start = yMd + " 23:55:00";
      const end = yMd + " 23:59:59";
      const data = await this.pooldb.exec(sql, [start, end]);
      if (data.length === 0) {
        return false;
      }
      const result = [];
      for (let item of data) {
        const { user, mhs24h } = item;
        result.push({ user, mhs24h: Number(mhs24h) });
      }
      return result;
    } catch (err) {
      throw err;
    }
  }

  async query_hashratedb_mhs24h(date) {
    try {
      const sql = `SELECT user, SUM(mhs24h) AS mhs24h FROM ${this.coin}_mhsv2 WHERE date = ? GROUP BY user;`;
      const data = await this.hashratedb.exec(sql, [date]);
      if (!data || data.length === 0) {
        return false;
      }
      const result = [];
      for (let item of data) {
        const { user, mhs24h } = item;
        result.push({ user, mhs24h: Number(mhs24h) });
      }
      return result;
    } catch (err) {
      throw err;
    }
  }

  async query_distribution_data(date) {
    try {
      const sql = `SELECT user, amount FROM wallet_in WHERE coin = ? AND create_date = ? AND user != "pool_account";`;
      const data = await this.distribution.exec(sql, [this.coin, date]);
      if (!data || data.length === 0) {
        return false;
      }
      const result = [];
      for (let item of data) {
        const { user, amount } = item;
        result.push({ user, amount: Number(amount) });
      }
      return result;
    } catch (err) {
      throw err;
    }
  }

  caculate_mhs24h_percent(data) {
    const totalAmount = data.reduce((acc, item) => acc + item.mhs24h, 0);
    const result = data.map((item) => ({
      user: item.user,
      mhs24h: item.mhs24h,
      percentage: Number((item.mhs24h / totalAmount).toFixed(4)), // 保留两位小数
    }));
    return result;
  }

  caculate_distribution_percent(data) {
    const totalAmount = data.reduce((acc, item) => acc + item.amount, 0);
    const result = data.map((item) => ({
      user: item.user,
      should_receive: Number((item.amount / 0.95).toFixed(8)),
      actual_receive: item.amount,
      percentage: Number(((item.amount / totalAmount)).toFixed(4)), // 保留两位小数
    }));
    return result;
  }

  async caculate_user_should_distribution(date) {
    try {
      const [user_mhs24h, user_distribution] = await Promise.all([this.query_hashratedb_mhs24h(date), this.query_distribution_data(date)]);
      const mhs24h_percent = this.caculate_mhs24h_percent(user_mhs24h)
      const distribution_percent = this.caculate_distribution_percent(user_distribution)
      let fees = 0
      for(let item of distribution_percent){
        const {user, should_receive, actual_receive, percentage} = item
        fees += should_receive - actual_receive
      }
      console.log(fees);
    } catch (err) {
      throw err;
    }
  }
}

module.exports = Stats;
